<?php
/*
<!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SMF DB Uniform Collation & Engine Utility
	Copyright 2023 Chen Zhen @ web-develop.ca
    Distributed under the terms of the GNU General Public License
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
*/
defined('SMF') OR exit('<b>Error:</b> SMF not defined');
function UniformCollations_subactions(&$subActions)
{
	global $user_info;
	if (!empty($user_info['is_admin'])) {
		$subActions += array(
			'uniformcollations' => array(
				'function' => 'UniformCollationsSettings',
				'activities' => array(),
			),
		);		
	}
}

function UniformCollationsSettings()
{
	global $txt, $scripturl, $modSettings, $sourcedir, $user_info, $settings, $context, $smcFunc, $db_name;

	if (empty($user_info['is_admin']))
		fatal_lang_error('UniformCollations_admin_error', false);

	require_once($sourcedir . '/Subs-Admin.php');
	require_once($sourcedir . '/Subs-Charset.php');
	require_once($sourcedir . '/ForceUTF8-Class.php');
	require_once($sourcedir . '/ForceCharConvert.php');
	$tableIdx = isset($_GET['table']) ? $_GET['table'] : -1;
	$_SESSION['uniform_tables_field_count'] = $tableIdx >= 0 && $tableIdx < count($_SESSION['uniform_tables']['tables']) ? GetUniformFieldsCount($_SESSION['uniform_tables']['tables'][$tableIdx]) : 0;
	$_SESSION['uniform_collation'] = !empty($_SESSION['uniform_collation']) ? $_SESSION['uniform_collation'] : '';
	$_SESSION['uniform_counter'] = !empty($_SESSION['uniform_counter']) ? abs($_SESSION['uniform_counter']) : 0;
	$_SESSION['uniform_field_counter'] = !empty($_SESSION['uniform_field_counter']) ? abs($_SESSION['uniform_field_counter']) : 0;
	$_SESSION['force_uniform_charcode'] = !empty($_SESSION['force_uniform_charcode']) ? $_SESSION['force_uniform_charcode'] : '';
	$fieldValues = !empty($_SESSION['uniform_tables']['fields']) ? $_SESSION['uniform_tables']['fields'] : array();
	$tables = !empty($_SESSION['uniform_tables']['tables']) ? $_SESSION['uniform_tables']['tables'] : array();
	$tableId = isset($_GET['table']) ? (int)$_GET['table'] : 0;
	$fieldCount = !empty($_SESSION['uniform_tables_field_count']) ? $_SESSION['uniform_tables_field_count'] : 0;
	list($collationSet, $tables, $rand) = array($txt['UniformCollations_config_unknown'], array(), rand(0, count($txt['UniformCollations_config_fields_messages'])-1));

	// the automatic db tasks should be finished prior to asking for any passwords repeatedly
	//$modSettings['securityDisable'] = 1;
	UniformCollationsSetCookie();
	ignore_user_abort(true);
	db_extend();

	// Query the SMF messages table to see if we can use utf8mb4
	$result = $smcFunc['db_query']('', 'SHOW TABLE STATUS LIKE "{db_prefix}messages"');
	while ($val = $smcFunc['db_fetch_assoc']($result))
		$collationSet = $val['Collation'];
	$smcFunc['db_free_result']($result);

	$context['html_headers'] .= '
		<link rel="stylesheet" href="' . $settings['default_theme_url'] . '/css/uniformcollations.css?fin23">
		<script type="text/javascript">
		function myDbChanges(myval)
		{
			var dbt = document.getElementById("dbtype");
			var OptedDbType = myval;
			var alertTextOutput = "' . $txt['UniformCollations_config_warning'] . '\n\nClick OK to confirm changing database settings to: " + OptedDbType;
			if (OptedDbType == "" || OptedDbType == "Select Collation")
				return false;
			var confirmChange = confirm(alertTextOutput);

			if (confirmChange == false)
			{
				dbt.value = "' . $collationSet . '";
				return false;
			}
			else
				return true;
		}
		function converterScheduledEvent()
		{
			var checkSave = document.getElementById("save_setting");
			' . (!empty($_REQUEST['uniform_counter']) || !empty($_REQUEST['uniform_field_counter']) ? 'collationWorkingHarder();' : 'checkSave.onclick = function(){collationWorking();};') . '
			' . (isset($_REQUEST['collations']) && $_REQUEST['collations'] == 'done' ? 'collationDone();' : '') . '
			' . (isset($_REQUEST['uniform_field_counter']) && isset($_GET['table']) ? 'collationFieldInfo();' : '') . '
		}
		function collationWorking()
		{
			var collationMsgExecuting = document.getElementById("collationMsg");
			collationFieldInfo2();
			collationMsgExecuting.style.display = "block";
			collationMsgExecuting.innerHTML = "' . $txt['UniformCollations_config_execute'] . '";
			collationMsgExecuting.style.height = "2em";
			var myinterval = setInterval(function(){
				collationMsgExecuting.innerHTML = (collationMsgExecuting.innerHTML == " " ? "' . $txt['UniformCollations_config_execute'] . '" : " ");
			}, 1000);
		}
		function collationWorkingHarder()
		{
			var collationMsgExecuting = document.getElementById("collationMsg");
			collationFieldInfo2();
			collationMsgExecuting.style.display = "block";
			collationMsgExecuting.innerHTML = "' . $txt['UniformCollations_config_execute_more'] . '";
			collationMsgExecuting.style.height = "2em";
			var myinterval = setInterval(function(){
				collationMsgExecuting.innerHTML = (collationMsgExecuting.innerHTML == " " ? "' . $txt['UniformCollations_config_execute_more'] . '" : " ");
			}, 1000);
		}
		function collationFieldInfo()
		{
			var collationMsgField = document.getElementById("collationMsgField");
			collationFieldInfo2();
			collationMsgField.style.display = "block";
			collationMsgField.innerHTML = "' . (!empty($tableId) && !empty($fieldValues) ? sprintf($txt['UniformCollations_config_field_count'], !empty($_SESSION['uniform_tables']['tables'][$_GET['table']]) ? $_SESSION['uniform_tables']['tables'][$_GET['table']] : $txt['UniformCollations_config_fields_void'], !empty($_GET['uniform_field_counter']) ? $_GET['uniform_field_counter'] . ' / ' . $fieldCount : $fieldCount) : " ") . '";
			collationMsgField.style.height = "2em";
		}
		function collationFieldInfo2()
		{
			var collationMsgFieldDelayHide = document.getElementById("collationMsgFieldDelay");
			collationMsgFieldDelayHide.innerHTML = "' . $txt['UniformCollations_config_fields_messages'][$rand] . '";
		}
		function collationHide()
		{
			var collationMsgHide = document.getElementById("collationMsg");
			var collationMsgFieldHide = document.getElementById("collationMsgField");
			var collationMsgFieldDelayHide = document.getElementById("collationMsgFieldDelay");
			collationMsgHide.innerHTML = " ";
			collationMsgFieldHide.innerHTML = " ";
			collationMsgFieldDelayHide.innerHTML = " ";
		}
		function collationDone()
		{
			var collationMsgDone = document.getElementById("collationMsg");
			collationMsgDone.style.display = "block";
			collationMsgDone.style.height = "2em";
			collationMsgDone.innerHTML = "' . $txt['UniformCollations_config_done'] . '";
			setTimeout(collationHide, 3000);
		}
		function collationSubmit()
		{
			setTimeout(function(){
				document.getElementById("save_setting").click();
				document.getElementById("collationform").submit();
			}, 2000);
		}
		if(window.attachEvent) {
			window.attachEvent("onload", converterScheduledEvent);
		}
		else {
			window.addEventListener("load", converterScheduledEvent, false);
		}
		</script>';

	/* Query Engine & Collation of the SMF database */
	$result = $smcFunc['db_query']('', "
		SHOW TABLE STATUS FROM {raw:db_name}",
		array('db_name' => $db_name)
	);

	while ($val = $smcFunc['db_fetch_assoc']($result))
	{
		$charsetx = explode('_', $val['Collation']);
		$collationSet = !empty($collationSet) ? $collationSet : $val['Collation'];
		$context['admin_current_db_values'] = array(
			'description' => !empty($txt[$val['Collation']]) ? $txt[$val['Collation']] : $txt['UniformCollations_config_unknown'],
			'engine' => $val['Engine'],
			'collation' => !empty($_SESSION['uniform_collation']) && !empty($_REQUEST['uniform_counter']) ? $_SESSION['uniform_collation'] : $val['Collation'],
			'charset' => $charsetx[0],
		);
	}
	$smcFunc['db_free_result']($result);

	$context['uniform_collations'] = UniformCollationsList();
	$collations = UniformCollationsList();

	if (isset($_REQUEST['save']))
	{
		//checkSession();		
		$collation = isset($_POST['db_collation']) ? $_POST['db_collation'] : (!empty($_SESSION['uniform_collation']) ? $_SESSION['uniform_collation'] : '');
		$_SESSION['uniform_collation'] = !empty($_SESSION['uniform_collation']) ? $_SESSION['uniform_collation'] : $collation;
		$_SESSION['uniform_charset'] = stripos($_SESSION['uniform_collation'], 'utf8mb4') !== false ? 'utf8mb4' : 'utf8';
		$forceCharset = !empty($_POST['force_charcode']) ? $_POST['force_charcode'] : (!empty($_SESSION['force_uniform_charcode']) ? $_SESSION['force_uniform_charcode'] : '');
		$adjustCollation = !empty($collation) ? UniformCollationsDatabase($context['admin_current_db_values'], $collationSet, $collation) : 0;
		$_SESSION['force_uniform_charcode'] = $forceCharset;

		switch ($forceCharset){
			case 'utf8mb4':
				$adjustCharset = UniformCollationsFixChars('utf8mb4');
				break;
			case 'utf8':
				$adjustCharset = UniformCollationsFixChars('utf8');
				break;
			case 'mb4utf8':
				$adjustCharset = UniformCollationsFixChars('mb4utf8');
				break;
			default:
				$adjustCharset = array(0, 0);
		}

		unset($_POST['save']);
		$context['html_headers'] .= '
		<script type="text/javascript">
			var collationMsg = document.getElementById("collationMsg");
			collationMsg.style.display = "block";
			collationMsg.innerHTML = "' . $txt['UniformCollations_config_execute'] . '";
			setTimeout(collationDone, 5000);
			setTimeout(collationHide, 5000);
		</script>';

		if (empty($forceCharset) && $adjustCollation > 0)
		{
			$context['admin_current_db_values']['collation'] = $_SESSION['uniform_collation'];
			redirectexit($scripturl . '?action=admin;area=maintain;sa=uniformcollations;uniform_counter=' . $adjustCollation . ';sesc=' . $context['session_id'] . ';#save_setting', true);
		}
		if (!empty($forceCharset))
		{
			$context['force_charset'] = $_SESSION['force_uniform_charcode'];
			redirectexit($scripturl . '?action=admin;area=maintain;sa=uniformcollations;table=' . $adjustCharset[0] . ';uniform_field_counter=' . $adjustCharset[1] . ';sesc=' . $context['session_id'] . ';#save_setting', true);
		}
		list($_SESSION['uniform_tables'], $_SESSION['uniform_field_counter'], $_SESSION['uniform_counter'], $_SESSION['uniform_collation'], $_SESSION['force_uniform_charcode']) = array(array(), 0, 0, '', '');
		redirectexit($scripturl . '?action=admin;area=maintain;sa=uniformcollations;collations=done;#save_setting');
	}

	if (isset($_REQUEST['collations']) && $_REQUEST['collations'] == 'done')
	{
		$result = $smcFunc['db_query']('',"
			SHOW TABLES FROM {raw:db_name}",
			array('db_name' => $db_name)
		);

		while ($row = $smcFunc['db_fetch_row']($result))
			$tables[] = $row[0];

		$smcFunc['db_free_result']($result);

		foreach ($tables as $table)
		{
			$alterTable = $smcFunc['db_query']('', "
					REPAIR TABLE {raw:db_name}.{raw:table}",
					array('db_name' => $db_name, 'table' => $table)
				);
			$alterTable = $smcFunc['db_query']('', "
					OPTIMIZE TABLE {raw:db_name}.{raw:table}",
					array('db_name' => $db_name, 'table' => $table)
				);
		}

		require_once($sourcedir . '/Subs-Charset.php');
		UniformCollations_fix_serialized_columns();
		list($_SESSION['uniform_tables'], $_SESSION['uniform_field_counter'], $_SESSION['uniform_counter'], $_SESSION['uniform_collation'], $_SESSION['force_uniform_charcode']) = array(array(), 0, 0, '', '');
	}
	elseif (!isset($_REQUEST['table']) && !isset($_REQUEST['uniform_field_counter']))
		list($_SESSION['uniform_tables'], $_SESSION['uniform_field_counter'], $_SESSION['uniform_counter'], $_SESSION['uniform_collation'], $_SESSION['force_uniform_charcode']) = array(array(), 0, 0, '', '');

	$context['force_charset'] = 0;
	$context['sub_template'] = 'edit_uniform_collations';
	$context['page_title'] = $txt['Uniform_Collations'];	
	$context['settings_title'] = $txt['Uniform_Collations'];
	$context['collation_addtoform'] = !empty($_REQUEST['table']) ? 'table=' . $_REQUEST['table'] . ';' : '';
	$context['collation_addtoform'] .= !empty($_REQUEST['uniform_counter']) ? 'uniform_counter=' . $_REQUEST['uniform_counter'] . ';' : '';
	$context['collation_addtoform'] .= !empty($_REQUEST['uniform_field_counter']) ? 'uniform_field_counter=' . $_REQUEST['uniform_field_counter'] . ';' : '';
	$context['post_url'] = $scripturl . '?action=admin;area=maintain;' . $context['collation_addtoform'] . 'sa=uniformcollations;save';
	loadTemplate('ManageUniformCollations');
}

function UniformCollationsFixChars($type = 'utf8')
{
	global $user_info, $txt;

	if (empty($user_info['is_admin']))
		fatal_lang_error('UniformCollations_admin_error_access', false);

	// Fix all db columns to use 3 byte UTF-8 characters
	$_SESSION['uniform_tables'] = !empty($_SESSION['uniform_tables']) ? $_SESSION['uniform_tables'] : array();
	if (empty($_SESSION['uniform_tables']['time']))
	{
		$_SESSION['uniform_tables']['time'] = time() + 300;
		$_SESSION['uniform_tables']['tables'] = GetAllNeededUniformTables();
		$_SESSION['uniform_tables']['fields'] = GetAllNeededUniformTablesFields($_SESSION['uniform_tables']['tables']);
		$_SESSION['uniform_tables_count'] = count($_SESSION['uniform_tables']['tables']);
	}
	elseif ($_SESSION['uniform_tables']['time'] - time() < 0)
	{
		$_SESSION['uniform_tables']['tables'] = GetAllNeededUniformTables();
		$_SESSION['uniform_tables']['fields'] = GetAllNeededUniformTablesFields($_SESSION['uniform_tables']['tables']);
		$_SESSION['uniform_tables']['time'] = time() + 300;
		$_SESSION['uniform_tables_count'] = count($_SESSION['uniform_tables']['tables']);
	}
	return ConvertDbColumns($type);
}

/* Adjust SMF tables to uniform Collation & Engine type*/
function UniformCollationsDatabase($adminValues, $collationSet, $dbtype = 'utf8_general_ci')
{
	global $user_info, $smcFunc, $context, $scripturl, $db_name, $txt, $db_user, $db_passwd;
	list($tables, $engine, $collation, $count) = array(array(), $adminValues['engine'], $dbtype, 0);
	$charset = stripos($collation, 'utf8mb4') === false ? 'utf8' : 'utf8mb4';
	$collations = UniformCollationsList();
	$count = isset($_GET['uniform_counter']) ? (int)$_GET['uniform_counter'] : $count;
	if (empty($user_info['is_admin']))
		fatal_lang_error('UniformCollations_admin_error_access', false);

	if (!in_array($dbtype, $collations))
		fatal_lang_error('UniformCollations_admin_error_collations', false);

	// $charset = $adminValues['charset'];
	// $charset = 'utf8';
	// $collation = 'utf8_general_ci';
	if (stripos($engine, 'innodb') !== false)
	{
		// causing error .. skipped
		/*
		$smcFunc['db_query']('', "			
			SET GLOBAL innodb_file_format={raw:bcformat};
			SET GLOBAL innodb_file_per_table={raw:pertable};
			SET GLOBAL innodb_large_prefix={raw:lgprefix};",
			array('bcformat' => 'Barracuda', 'pertable' => 'on', 'lgprefix' => 'on')
		);
		*/
	}

	/* Query all tables */
	$result = $smcFunc['db_query']('',"
		SHOW TABLES FROM {raw:db_name}",
		array('db_name' => $db_name)
	);

	while ($row = $smcFunc['db_fetch_row']($result))
    	$tables[] = $row[0];

	$smcFunc['db_free_result']($result);

	$alterDatabase = $smcFunc['db_query']('', "
		ALTER DATABASE {raw:dbname} CHARACTER SET = {raw:charset} COLLATE = {raw:collation}",
			array('dbname' => $db_name, 'charset' => $charset, 'collation' => $collation)
	);

	$_SESSION['uniform_tables_count'] = count($tables) > 0 ? count($tables)-1 : 0;
	$_SESSION['uniform_tables'] = $tables;
	if (!empty($count))
	{
		for ($i=0;$i<$count;$i++)
			unset($tables[$i]);

		$tables = array_values($tables);
	}
	elseif (empty($tables))
	{
		$_SESSION['uniform_counter'] = 0;
		return -1;
	}

	foreach ($tables as $table)
	{
		if (!$table)
			continue;

		if (stripos($charset, 'utf8mb4') !== false)
		{
			$alterTable = $smcFunc['db_query']('', "
				ALTER TABLE {raw:db_name}.{raw:table}
				ROW_FORMAT=DYNAMIC",
				array('db_name' => $db_name, 'table' => $table)
			);

			UniformCollationsUtf8mb4($table, $dbtype);

			$alterTable = $smcFunc['db_query']('', "
				ALTER TABLE {raw:db_name}.{raw:table}
				CONVERT TO CHARACTER SET {raw:charset}
				COLLATE {raw:collation}",
				array('db_name' => $db_name, 'table' => $table, 'charset' => $charset, 'collation' => $collation)
			);
		}
		else
		{
			UniformCollationsUtf8($table, $dbtype);

			$alterTable = $smcFunc['db_query']('', "
				ALTER TABLE {raw:db_name}.{raw:table}
				CONVERT TO CHARACTER SET {raw:charset}
				COLLATE {raw:collation}",
				array('db_name' => $db_name, 'table' => $table, 'charset' => $charset, 'collation' => $collation)
			);
		}

		if (function_exists('apache_reset_timeout'))
			@apache_reset_timeout();

		$count++;

		// 15 tables at a time
		if ($count % 15 == 0)
			return $count;
	}

	list($_SESSION['uniform_collation'], $_SESSION['uniform_counter']) = array('', 0);
	return 0;
}

function UniformCollationsUtf8($table, $dbtype = 'utf8_general_ci')
{
	global $smcFunc, $db_name;
	db_extend('packages');
	list($columns, $medBytes, $charset, $x) = array(array(), 16777215, 'utf8', 0);

	$request = $smcFunc['db_query']('', "
		SHOW FULL COLUMNS
		FROM {raw:db}.{raw:table}",
		array('table' => $table, 'db' => $db_name,)
	);
	while ($column_info = $smcFunc['db_fetch_assoc']($request))
	{
		if (stripos($column_info['Type'], 'text') !== false || stripos($column_info['Type'], 'char') !== false)
			$columns[] = $column_info;
	}

	$smcFunc['db_free_result']($request);

	foreach ($columns as $column)
	{
		$result = $smcFunc['db_query']('','
			SELECT CHARACTER_OCTET_LENGTH, CHARACTER_MAXIMUM_LENGTH
			FROM information_schema.columns
			WHERE table_schema = "{raw:db}" AND table_name="{raw:tablex}" AND column_name="{raw:columnx}"',
			array('db' => $db_name, 'tablex' => $table, 'columnx' => $column['Field'])
		);

		while ($row = $smcFunc['db_fetch_row']($result))
		{
		   	$columns[$x]['Length'] = (int)$row[1];
			$columns[$x]['Octet'] = (int)$row[0];
		}

		$x++;
	}
	if (!empty($columns))
		$smcFunc['db_free_result']($result);

	foreach ($columns as $column)
	{
		// only VARCHAR are changed. TEXT types are left alone and will have to remain with a larger byte allowance if converting from UTF8MB4
		if (stripos($column['Type'], 'varchar') !== false && $column['Length'] == 191)
		{
			$length = 255;
			$alterTable = $smcFunc['db_query']('', "
				ALTER TABLE {raw:db_name}.{raw:table}
				CHANGE {raw:column} {raw:column} VARCHAR(" . $length . ") CHARACTER SET {raw:charset} COLLATE {raw:dbtype}",
				array('db_name' => $db_name, 'table' => $table, 'charset' => $charset, 'collation' => $dbtype, 'column' => $column['Field'], 'dbtype' => $dbtype, 'length' => $length)
			);
		}

		if (function_exists('apache_reset_timeout'))
			@apache_reset_timeout();
	}
}

function UniformCollationsUtf8mb4($table, $dbtype = 'utf8mb4_general_ci')
{
	global $smcFunc, $db_name, $user_info, $txt;
	db_extend('packages');
	list($columns, $maxBytes, $charset, $x) = array(array(), 65535, 'utf8mb4', 0);

	if (empty($user_info['is_admin']))
		fatal_lang_error('UniformCollations_admin_error_access', false);

	$request = $smcFunc['db_query']('', "
		SHOW FULL COLUMNS
		FROM {raw:db}.{raw:table}",
		array('table' => $table, 'db' => $db_name,)
	);
	while ($column_info = $smcFunc['db_fetch_assoc']($request))
	{
		if (stripos($column_info['Type'], 'text') !== false || stripos($column_info['Type'], 'char') !== false)
			$columns[] = $column_info;
	}

	$smcFunc['db_free_result']($request);

	foreach ($columns as $column)
	{
		$result = $smcFunc['db_query']('','
			SELECT CHARACTER_OCTET_LENGTH, CHARACTER_MAXIMUM_LENGTH
			FROM information_schema.columns
			WHERE table_schema = "{raw:db}" AND table_name="{raw:tablex}" AND column_name="{raw:columnx}"',
			array('db' => $db_name, 'tablex' => $table, 'columnx' => $column['Field'])
		);

		while ($row = $smcFunc['db_fetch_row']($result))
		   	$columns[$x]['Length'] = (int)$row[1];

		$x++;
	}
	if (!empty($columns))
		$smcFunc['db_free_result']($result);

	foreach ($columns as $column)
	{
		if (stripos($column['Type'], 'varchar') !== false && $column['Length'] > 0 && $column['Length'] < 256)
		{
			$length = $column['Length'] > 191 ? 191 : $column['Length'];
			$alterTable = $smcFunc['db_query']('', "
				ALTER TABLE {raw:db_name}.{raw:table}
				CHANGE {raw:column} {raw:column} VARCHAR(" . $length . ") CHARACTER SET {raw:charset} COLLATE {raw:dbtype}",
				array('db_name' => $db_name, 'table' => $table, 'charset' => $charset, 'collation' => $dbtype, 'column' => $column['Field'], 'dbtype' => $dbtype, 'length' => $length)
			);
		}
		elseif (stripos($column['Type'], 'text') !== false && $column['Length'] < $maxBytes)
		{
			$alterTable = $smcFunc['db_query']('', "
				ALTER TABLE {raw:db_name}.{raw:table}
				CHANGE {raw:column} {raw:column} TEXT CHARACTER SET {raw:charset} COLLATE {raw:dbtype}",
				array('db_name' => $db_name, 'table' => $table, 'charset' => $charset, 'collation' => $dbtype, 'column' => $column['Field'], 'dbtype' => $dbtype)
			);
		}
		elseif (stripos($column['Type'], 'text') !== false && $column['Length'] >= $maxBytes)
		{
			$alterTable = $smcFunc['db_query']('', "
				ALTER TABLE {raw:db_name}.{raw:table}
				CHANGE {raw:column} {raw:column} MEDIUMTEXT CHARACTER SET {raw:charset} COLLATE {raw:dbtype}",
				array('db_name' => $db_name, 'table' => $table, 'charset' => $charset, 'collation' => $dbtype, 'column' => $column['Field'], 'dbtype' => $dbtype)
			);
		}
	}
}

function UniformCollations_admin_areas(&$admin_areas)
{
	global $txt;

	$admin_areas['maintenance']['areas']['maintain']['subsections'] += array(
		'uniformcollations' => array($txt['Uniform_Collations'], 'admin_forum'),
	);
}


function UniformCollations_init()
{
	// load necessary language files
	global $txt;
	loadLanguage('UniformCollations');
}

function UniformCollationsSetCookie()
{
	// this is necessary due to possible lengthy database updates
	global $cookiename, $user_info, $user_settings, $modSettings, $sourcedir;
	require_once($sourcedir . '/Subs-Auth.php');

	$v = version_compare((!empty($modSettings['smfVersion']) ? substr($modSettings['smfVersion'], 0, 3) : '2.0'), '2.1', '<') ? 'v2.0' : 'v2.1';
	$cookieTime = !empty($modSettings['cookieTime']) ? $modSettings['cookieTime'] * 60 : 3600;
	list ($a, $b, $timeout) = !empty($_COOKIE[$cookiename]) && UniformCollations_is_serialized($_COOKIE[$cookiename]) ? safe_unserialize($_COOKIE[$cookiename]) : array('', '', (time() + $cookieTime));

	if ($v == 'v2.0')
		setLoginCookie($timeout - time(), $user_info['id'], sha1($user_settings['passwd'] . $user_settings['password_salt']));
	else
		setLoginCookie((int) $timeout - time(), $user_info['id'], hash_salt($user_settings['passwd'], $user_settings['password_salt']));
}

function UniformCollations_is_serialized ($data, $strict = true)
{
	if ( ! is_string( $data ) ) {
		return false;
	}
	$data = trim( $data );
	if ( 'N;' === $data ) {
		return true;
	}
	if ( strlen( $data ) < 4 ) {
		return false;
	}
	if ( ':' !== $data[1] ) {
		return false;
	}
	if ( $strict ) {
		$lastc = substr( $data, -1 );
		if ( ';' !== $lastc && '}' !== $lastc ) {
			return false;
		}
	} else {
		$semicolon = strpos( $data, ';' );
		$brace     = strpos( $data, '}' );
		if ( false === $semicolon && false === $brace ) {
			return false;
		}
		if ( false !== $semicolon && $semicolon < 3 ) {
			return false;
		}
		if ( false !== $brace && $brace < 4 ) {
			return false;
		}
	}
	$token = $data[0];
	switch ( $token ) {
		case 's':
			if ( $strict ) {
				if ( '"' !== substr( $data, -2, 1 ) ) {
					return false;
				}
			} elseif ( false === strpos( $data, '"' ) ) {
				return false;
			}
		case 'a':
		case 'O':
		case 'E':
			return (bool) preg_match( "/^{$token}:[0-9]+:/s", $data );
		case 'b':
		case 'i':
		case 'd':
			$end = $strict ? '$' : '';
			return (bool) preg_match( "/^{$token}:[0-9.E+-]+;$end/", $data );
	}
	return false;
}

function UniformCollationsList()
{
	$collations['utf8'] = array(
		'utf8_bin',
		'utf8_croatian_ci',
		'utf8_croatian_mysql561_ci',
		'utf8_czech_ci',
		'utf8_danish_ci',
		'utf8_esperanto_ci',
		'utf8_estonian_ci',
		'utf8_general_ci',
		'utf8_general_mysql500_ci',
		'utf8_general_nopad_ci',
		'utf8_german2_ci',
		'utf8_hungarian_ci',
		'utf8_icelandic_ci',
		'utf8_latvian_ci',
		'utf8_lithuanian_ci',
		'utf8_myanmar_ci',
		'utf8_nopad_bin',
		'utf8_persian_ci',
		'utf8_polish_ci',
		'utf8_romanian_ci',
		'utf8_sinhala_ci',
		'utf8_slovak_ci',
		'utf8_slovenian_ci',
		'utf8_spanish2_ci',
		'utf8_spanish_ci',
		'utf8_swedish_ci',
		'utf8_thai_520_w2',
		'utf8_turkish_ci',
		'utf8_unicode_520_ci',
		'utf8_unicode_520_nopad_ci',
		'utf8_unicode_ci',
		'utf8_unicode_nopad_ci',
		'utf8_vietnamese_ci',
	);

	$collations['utf8mb4'] = array(
		'utf8mb4_bin',
		'utf8mb4_croatian_ci',
		'utf8mb4_croatian_mysql561_ci',
		'utf8mb4_czech_ci',
		'utf8mb4_danish_ci',
		'utf8mb4_esperanto_ci',
		'utf8mb4_estonian_ci',
		'utf8mb4_general_ci',
		'utf8mb4_general_mysql500_ci',
		'utf8mb4_general_nopad_ci',
		'utf8mb4_german2_ci',
		'utf8mb4_hungarian_ci',
		'utf8mb4_icelandic_ci',
		'utf8mb4_latvian_ci',
		'utf8mb4_lithuanian_ci',
		'utf8mb4_myanmar_ci',
		'utf8mb4_nopad_bin',
		'utf8mb4_persian_ci',
		'utf8mb4_polish_ci',
		'utf8mb4_romanian_ci',
		'utf8mb4_sinhala_ci',
		'utf8mb4_slovak_ci',
		'utf8mb4_slovenian_ci',
		'utf8mb4_spanish2_ci',
		'utf8mb4_spanish_ci',
		'utf8mb4_swedish_ci',
		'utf8mb4_thai_520_w2',
		'utf8mb4_turkish_ci',
		'utf8mb4_unicode_520_ci',
		'utf8mb4_unicode_520_nopad_ci',
		'utf8mb4_unicode_ci',
		'utf8mb4_unicode_nopad_ci',
		'utf8mb4_vietnamese_ci',
	);

	return array_merge($collations['utf8'], $collations['utf8mb4']);
}

function UniformCollations_fix_serialized_columns()
{
	global $smcFunc;

	$request = $smcFunc['db_query']('', '
		SELECT id_action, extra
		FROM {db_prefix}log_actions
		WHERE action IN ({string:remove}, {string:delete})',
		array(
			'remove' => 'remove',
			'delete' => 'delete',
		)
	);
	while ($row = $smcFunc['db_fetch_assoc']($request))
	{
		if (@safe_unserialize($row['extra']) === false && preg_match('~^(a:3:{s:5:"topic";i:\d+;s:7:"subject";s:)(\d+):"(.+)"(;s:6:"member";s:5:"\d+";})$~', $row['extra'], $matches) === 1)
			$smcFunc['db_query']('', '
				UPDATE {db_prefix}log_actions
				SET extra = {string:extra}
				WHERE id_action = {int:current_action}',
				array(
					'current_action' => $row['id_action'],
					'extra' => $matches[1] . strlen($matches[3]) . ':"' . $matches[3] . '"' . $matches[4],
				)
			);
	}
	$smcFunc['db_free_result']($request);

	// Refresh some cached data.
	updateSettings(array(
		'memberlist_updated' => time(),
	));

}

?>